package Samba::Security::Descriptor;

use 5.014002;
use strict;
use warnings;
use Carp;
use XS::Object::Magic;

require Exporter;
use AutoLoader;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Samba::Security::Descriptor ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	BUILTIN_RID_ACCOUNT_OPERATORS
	BUILTIN_RID_ADMINISTRATORS
	BUILTIN_RID_AUTH_ACCESS
	BUILTIN_RID_BACKUP_OPERATORS
	BUILTIN_RID_CERT_SERV_DCOM_ACCESS
	BUILTIN_RID_CRYPTO_OPERATORS
	BUILTIN_RID_DISTRIBUTED_COM_USERS
	BUILTIN_RID_EVENT_LOG_READERS
	BUILTIN_RID_GUESTS
	BUILTIN_RID_INCOMING_FOREST_TRUST
	BUILTIN_RID_NETWORK_CONF_OPERATORS
	BUILTIN_RID_PERFLOG_USERS
	BUILTIN_RID_PERFMON_USERS
	BUILTIN_RID_POWER_USERS
	BUILTIN_RID_PRE_2K_ACCESS
	BUILTIN_RID_PRINT_OPERATORS
	BUILTIN_RID_RAS_SERVERS
	BUILTIN_RID_REMOTE_DESKTOP_USERS
	BUILTIN_RID_REPLICATOR
	BUILTIN_RID_SERVER_OPERATORS
	BUILTIN_RID_TS_LICENSE_SERVERS
	BUILTIN_RID_USERS
	DOMAIN_RID_ADMINISTRATOR
	DOMAIN_RID_ADMINS
	DOMAIN_RID_CERT_ADMINS
	DOMAIN_RID_DCS
	DOMAIN_RID_DOMAIN_MEMBERS
	DOMAIN_RID_ENTERPRISE_ADMINS
	DOMAIN_RID_ENTERPRISE_READONLY_DCS
	DOMAIN_RID_GUEST
	DOMAIN_RID_GUESTS
	DOMAIN_RID_KRBTGT
	DOMAIN_RID_LOGON
	DOMAIN_RID_POLICY_ADMINS
	DOMAIN_RID_RAS_SERVERS
	DOMAIN_RID_READONLY_DCS
	DOMAIN_RID_RODC_ALLOW
	DOMAIN_RID_RODC_DENY
	DOMAIN_RID_SCHEMA_ADMINS
	DOMAIN_RID_USERS
	KERB_ENCTYPE_AES128_CTS_HMAC_SHA1_96
	KERB_ENCTYPE_AES256_CTS_HMAC_SHA1_96
	KERB_ENCTYPE_DES_CBC_CRC
	KERB_ENCTYPE_DES_CBC_MD5
	KERB_ENCTYPE_RC4_HMAC_MD5
	LSA_POLICY_MODE_ALL
	LSA_POLICY_MODE_ALL_NT4
	LSA_POLICY_MODE_BATCH
	LSA_POLICY_MODE_DENY_BATCH
	LSA_POLICY_MODE_DENY_INTERACTIVE
	LSA_POLICY_MODE_DENY_NETWORK
	LSA_POLICY_MODE_DENY_REMOTE_INTERACTIVE
	LSA_POLICY_MODE_DENY_SERVICE
	LSA_POLICY_MODE_INTERACTIVE
	LSA_POLICY_MODE_NETWORK
	LSA_POLICY_MODE_PROXY
	LSA_POLICY_MODE_REMOTE_INTERACTIVE
	LSA_POLICY_MODE_SERVICE
	NT4_ACL_REVISION
	SD_REVISION
	SECINFO_DACL
	SECINFO_GROUP
	SECINFO_LABEL
	SECINFO_OWNER
	SECINFO_PROTECTED_DACL
	SECINFO_PROTECTED_SACL
	SECINFO_SACL
	SECINFO_UNPROTECTED_DACL
	SECINFO_UNPROTECTED_SACL
	SECURITY_ACL_REVISION_ADS
	SECURITY_ACL_REVISION_NT4
	SECURITY_DESCRIPTOR_REVISION_1
	SEC_ACE_FLAG_CONTAINER_INHERIT
	SEC_ACE_FLAG_FAILED_ACCESS
	SEC_ACE_FLAG_INHERITED_ACE
	SEC_ACE_FLAG_INHERIT_ONLY
	SEC_ACE_FLAG_NO_PROPAGATE_INHERIT
	SEC_ACE_FLAG_OBJECT_INHERIT
	SEC_ACE_FLAG_SUCCESSFUL_ACCESS
	SEC_ACE_FLAG_VALID_INHERIT
	SEC_ACE_INHERITED_OBJECT_TYPE_PRESENT
	SEC_ACE_OBJECT_TYPE_PRESENT
	SEC_ACE_TYPE_ACCESS_ALLOWED
	SEC_ACE_TYPE_ACCESS_ALLOWED_OBJECT
	SEC_ACE_TYPE_ACCESS_DENIED
	SEC_ACE_TYPE_ACCESS_DENIED_OBJECT
	SEC_ACE_TYPE_ALLOWED_COMPOUND
	SEC_ACE_TYPE_SYSTEM_ALARM
	SEC_ACE_TYPE_SYSTEM_ALARM_OBJECT
	SEC_ACE_TYPE_SYSTEM_AUDIT
	SEC_ACE_TYPE_SYSTEM_AUDIT_OBJECT
	SEC_ADS_CONTROL_ACCESS
	SEC_ADS_CREATE_CHILD
	SEC_ADS_DELETE_CHILD
	SEC_ADS_DELETE_TREE
	SEC_ADS_GENERIC_ALL
	SEC_ADS_GENERIC_ALL_DS
	SEC_ADS_GENERIC_EXECUTE
	SEC_ADS_GENERIC_READ
	SEC_ADS_GENERIC_WRITE
	SEC_ADS_LIST
	SEC_ADS_LIST_OBJECT
	SEC_ADS_READ_PROP
	SEC_ADS_SELF_WRITE
	SEC_ADS_WRITE_PROP
	SEC_DACL_AUTO_INHERIT
	SEC_DEFAULT_DESCRIPTOR
	SEC_DESC_DACL_AUTO_INHERITED
	SEC_DESC_DACL_AUTO_INHERIT_REQ
	SEC_DESC_DACL_DEFAULTED
	SEC_DESC_DACL_PRESENT
	SEC_DESC_DACL_PROTECTED
	SEC_DESC_DACL_TRUSTED
	SEC_DESC_GROUP_DEFAULTED
	SEC_DESC_OWNER_DEFAULTED
	SEC_DESC_RM_CONTROL_VALID
	SEC_DESC_SACL_AUTO_INHERITED
	SEC_DESC_SACL_AUTO_INHERIT_REQ
	SEC_DESC_SACL_DEFAULTED
	SEC_DESC_SACL_PRESENT
	SEC_DESC_SACL_PROTECTED
	SEC_DESC_SELF_RELATIVE
	SEC_DESC_SERVER_SECURITY
	SEC_DIR_ADD_FILE
	SEC_DIR_ADD_SUBDIR
	SEC_DIR_DELETE_CHILD
	SEC_DIR_LIST
	SEC_DIR_READ_ATTRIBUTE
	SEC_DIR_READ_EA
	SEC_DIR_TRAVERSE
	SEC_DIR_WRITE_ATTRIBUTE
	SEC_DIR_WRITE_EA
	SEC_FILE_ALL
	SEC_FILE_APPEND_DATA
	SEC_FILE_EXECUTE
	SEC_FILE_READ_ATTRIBUTE
	SEC_FILE_READ_DATA
	SEC_FILE_READ_EA
	SEC_FILE_WRITE_ATTRIBUTE
	SEC_FILE_WRITE_DATA
	SEC_FILE_WRITE_EA
	SEC_FLAG_MAXIMUM_ALLOWED
	SEC_FLAG_SYSTEM_SECURITY
	SEC_GENERIC_ALL
	SEC_GENERIC_EXECUTE
	SEC_GENERIC_READ
	SEC_GENERIC_WRITE
	SEC_GROUP_FROM_PARENT
	SEC_MASK_FLAGS
	SEC_MASK_GENERIC
	SEC_MASK_INVALID
	SEC_MASK_SPECIFIC
	SEC_MASK_STANDARD
	SEC_OWNER_FROM_PARENT
	SEC_PRIV_ADD_USERS
	SEC_PRIV_ADD_USERS_BIT
	SEC_PRIV_BACKUP
	SEC_PRIV_BACKUP_BIT
	SEC_PRIV_CHANGE_NOTIFY
	SEC_PRIV_CHANGE_NOTIFY_BIT
	SEC_PRIV_CREATE_GLOBAL
	SEC_PRIV_CREATE_GLOBAL_BIT
	SEC_PRIV_CREATE_PAGEFILE
	SEC_PRIV_CREATE_PAGEFILE_BIT
	SEC_PRIV_DEBUG
	SEC_PRIV_DEBUG_BIT
	SEC_PRIV_DISK_OPERATOR
	SEC_PRIV_DISK_OPERATOR_BIT
	SEC_PRIV_ENABLE_DELEGATION
	SEC_PRIV_ENABLE_DELEGATION_BIT
	SEC_PRIV_IMPERSONATE
	SEC_PRIV_IMPERSONATE_BIT
	SEC_PRIV_INCREASE_BASE_PRIORITY
	SEC_PRIV_INCREASE_BASE_PRIORITY_BIT
	SEC_PRIV_INCREASE_QUOTA
	SEC_PRIV_INCREASE_QUOTA_BIT
	SEC_PRIV_INVALID
	SEC_PRIV_LOAD_DRIVER
	SEC_PRIV_LOAD_DRIVER_BIT
	SEC_PRIV_MACHINE_ACCOUNT
	SEC_PRIV_MACHINE_ACCOUNT_BIT
	SEC_PRIV_MANAGE_VOLUME
	SEC_PRIV_MANAGE_VOLUME_BIT
	SEC_PRIV_PRINT_OPERATOR
	SEC_PRIV_PRINT_OPERATOR_BIT
	SEC_PRIV_PROFILE_SINGLE_PROCESS
	SEC_PRIV_PROFILE_SINGLE_PROCESS_BIT
	SEC_PRIV_REMOTE_SHUTDOWN
	SEC_PRIV_REMOTE_SHUTDOWN_BIT
	SEC_PRIV_RESTORE
	SEC_PRIV_RESTORE_BIT
	SEC_PRIV_SECURITY
	SEC_PRIV_SECURITY_BIT
	SEC_PRIV_SHUTDOWN
	SEC_PRIV_SHUTDOWN_BIT
	SEC_PRIV_SYSTEMTIME
	SEC_PRIV_SYSTEMTIME_BIT
	SEC_PRIV_SYSTEM_ENVIRONMENT
	SEC_PRIV_SYSTEM_ENVIRONMENT_BIT
	SEC_PRIV_SYSTEM_PROFILE
	SEC_PRIV_SYSTEM_PROFILE_BIT
	SEC_PRIV_TAKE_OWNERSHIP
	SEC_PRIV_TAKE_OWNERSHIP_BIT
	SEC_PRIV_UNDOCK
	SEC_PRIV_UNDOCK_BIT
	SEC_REG_CREATE_LINK
	SEC_REG_CREATE_SUBKEY
	SEC_REG_ENUM_SUBKEYS
	SEC_REG_NOTIFY
	SEC_REG_QUERY_VALUE
	SEC_REG_SET_VALUE
	SEC_RIGHTS_DIR_ALL
	SEC_RIGHTS_DIR_EXECUTE
	SEC_RIGHTS_DIR_READ
	SEC_RIGHTS_DIR_WRITE
	SEC_RIGHTS_FILE_ALL
	SEC_RIGHTS_FILE_EXECUTE
	SEC_RIGHTS_FILE_READ
	SEC_RIGHTS_FILE_WRITE
	SEC_RIGHTS_PRIV_BACKUP
	SEC_RIGHTS_PRIV_RESTORE
	SEC_SACL_AUTO_INHERIT
	SEC_STD_ALL
	SEC_STD_DELETE
	SEC_STD_READ_CONTROL
	SEC_STD_REQUIRED
	SEC_STD_SYNCHRONIZE
	SEC_STD_WRITE_DAC
	SEC_STD_WRITE_OWNER
	STANDARD_RIGHTS_ALL_ACCESS
	STANDARD_RIGHTS_EXECUTE_ACCESS
	STANDARD_RIGHTS_MODIFY_ACCESS
	STANDARD_RIGHTS_READ_ACCESS
	STANDARD_RIGHTS_REQUIRED_ACCESS
	STANDARD_RIGHTS_WRITE_ACCESS
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	BUILTIN_RID_ACCOUNT_OPERATORS
	BUILTIN_RID_ADMINISTRATORS
	BUILTIN_RID_AUTH_ACCESS
	BUILTIN_RID_BACKUP_OPERATORS
	BUILTIN_RID_CERT_SERV_DCOM_ACCESS
	BUILTIN_RID_CRYPTO_OPERATORS
	BUILTIN_RID_DISTRIBUTED_COM_USERS
	BUILTIN_RID_EVENT_LOG_READERS
	BUILTIN_RID_GUESTS
	BUILTIN_RID_INCOMING_FOREST_TRUST
	BUILTIN_RID_NETWORK_CONF_OPERATORS
	BUILTIN_RID_PERFLOG_USERS
	BUILTIN_RID_PERFMON_USERS
	BUILTIN_RID_POWER_USERS
	BUILTIN_RID_PRE_2K_ACCESS
	BUILTIN_RID_PRINT_OPERATORS
	BUILTIN_RID_RAS_SERVERS
	BUILTIN_RID_REMOTE_DESKTOP_USERS
	BUILTIN_RID_REPLICATOR
	BUILTIN_RID_SERVER_OPERATORS
	BUILTIN_RID_TS_LICENSE_SERVERS
	BUILTIN_RID_USERS
	DOMAIN_RID_ADMINISTRATOR
	DOMAIN_RID_ADMINS
	DOMAIN_RID_CERT_ADMINS
	DOMAIN_RID_DCS
	DOMAIN_RID_DOMAIN_MEMBERS
	DOMAIN_RID_ENTERPRISE_ADMINS
	DOMAIN_RID_ENTERPRISE_READONLY_DCS
	DOMAIN_RID_GUEST
	DOMAIN_RID_GUESTS
	DOMAIN_RID_KRBTGT
	DOMAIN_RID_LOGON
	DOMAIN_RID_POLICY_ADMINS
	DOMAIN_RID_RAS_SERVERS
	DOMAIN_RID_READONLY_DCS
	DOMAIN_RID_RODC_ALLOW
	DOMAIN_RID_RODC_DENY
	DOMAIN_RID_SCHEMA_ADMINS
	DOMAIN_RID_USERS
	KERB_ENCTYPE_AES128_CTS_HMAC_SHA1_96
	KERB_ENCTYPE_AES256_CTS_HMAC_SHA1_96
	KERB_ENCTYPE_DES_CBC_CRC
	KERB_ENCTYPE_DES_CBC_MD5
	KERB_ENCTYPE_RC4_HMAC_MD5
	LSA_POLICY_MODE_ALL
	LSA_POLICY_MODE_ALL_NT4
	LSA_POLICY_MODE_BATCH
	LSA_POLICY_MODE_DENY_BATCH
	LSA_POLICY_MODE_DENY_INTERACTIVE
	LSA_POLICY_MODE_DENY_NETWORK
	LSA_POLICY_MODE_DENY_REMOTE_INTERACTIVE
	LSA_POLICY_MODE_DENY_SERVICE
	LSA_POLICY_MODE_INTERACTIVE
	LSA_POLICY_MODE_NETWORK
	LSA_POLICY_MODE_PROXY
	LSA_POLICY_MODE_REMOTE_INTERACTIVE
	LSA_POLICY_MODE_SERVICE
	NT4_ACL_REVISION
	SD_REVISION
	SECINFO_DACL
	SECINFO_GROUP
	SECINFO_LABEL
	SECINFO_OWNER
	SECINFO_PROTECTED_DACL
	SECINFO_PROTECTED_SACL
	SECINFO_SACL
	SECINFO_UNPROTECTED_DACL
	SECINFO_UNPROTECTED_SACL
	SECURITY_ACL_REVISION_ADS
	SECURITY_ACL_REVISION_NT4
	SECURITY_DESCRIPTOR_REVISION_1
	SEC_ACE_FLAG_CONTAINER_INHERIT
	SEC_ACE_FLAG_FAILED_ACCESS
	SEC_ACE_FLAG_INHERITED_ACE
	SEC_ACE_FLAG_INHERIT_ONLY
	SEC_ACE_FLAG_NO_PROPAGATE_INHERIT
	SEC_ACE_FLAG_OBJECT_INHERIT
	SEC_ACE_FLAG_SUCCESSFUL_ACCESS
	SEC_ACE_FLAG_VALID_INHERIT
	SEC_ACE_INHERITED_OBJECT_TYPE_PRESENT
	SEC_ACE_OBJECT_TYPE_PRESENT
	SEC_ACE_TYPE_ACCESS_ALLOWED
	SEC_ACE_TYPE_ACCESS_ALLOWED_OBJECT
	SEC_ACE_TYPE_ACCESS_DENIED
	SEC_ACE_TYPE_ACCESS_DENIED_OBJECT
	SEC_ACE_TYPE_ALLOWED_COMPOUND
	SEC_ACE_TYPE_SYSTEM_ALARM
	SEC_ACE_TYPE_SYSTEM_ALARM_OBJECT
	SEC_ACE_TYPE_SYSTEM_AUDIT
	SEC_ACE_TYPE_SYSTEM_AUDIT_OBJECT
	SEC_ADS_CONTROL_ACCESS
	SEC_ADS_CREATE_CHILD
	SEC_ADS_DELETE_CHILD
	SEC_ADS_DELETE_TREE
	SEC_ADS_GENERIC_ALL
	SEC_ADS_GENERIC_ALL_DS
	SEC_ADS_GENERIC_EXECUTE
	SEC_ADS_GENERIC_READ
	SEC_ADS_GENERIC_WRITE
	SEC_ADS_LIST
	SEC_ADS_LIST_OBJECT
	SEC_ADS_READ_PROP
	SEC_ADS_SELF_WRITE
	SEC_ADS_WRITE_PROP
	SEC_DACL_AUTO_INHERIT
	SEC_DEFAULT_DESCRIPTOR
	SEC_DESC_DACL_AUTO_INHERITED
	SEC_DESC_DACL_AUTO_INHERIT_REQ
	SEC_DESC_DACL_DEFAULTED
	SEC_DESC_DACL_PRESENT
	SEC_DESC_DACL_PROTECTED
	SEC_DESC_DACL_TRUSTED
	SEC_DESC_GROUP_DEFAULTED
	SEC_DESC_OWNER_DEFAULTED
	SEC_DESC_RM_CONTROL_VALID
	SEC_DESC_SACL_AUTO_INHERITED
	SEC_DESC_SACL_AUTO_INHERIT_REQ
	SEC_DESC_SACL_DEFAULTED
	SEC_DESC_SACL_PRESENT
	SEC_DESC_SACL_PROTECTED
	SEC_DESC_SELF_RELATIVE
	SEC_DESC_SERVER_SECURITY
	SEC_DIR_ADD_FILE
	SEC_DIR_ADD_SUBDIR
	SEC_DIR_DELETE_CHILD
	SEC_DIR_LIST
	SEC_DIR_READ_ATTRIBUTE
	SEC_DIR_READ_EA
	SEC_DIR_TRAVERSE
	SEC_DIR_WRITE_ATTRIBUTE
	SEC_DIR_WRITE_EA
	SEC_FILE_ALL
	SEC_FILE_APPEND_DATA
	SEC_FILE_EXECUTE
	SEC_FILE_READ_ATTRIBUTE
	SEC_FILE_READ_DATA
	SEC_FILE_READ_EA
	SEC_FILE_WRITE_ATTRIBUTE
	SEC_FILE_WRITE_DATA
	SEC_FILE_WRITE_EA
	SEC_FLAG_MAXIMUM_ALLOWED
	SEC_FLAG_SYSTEM_SECURITY
	SEC_GENERIC_ALL
	SEC_GENERIC_EXECUTE
	SEC_GENERIC_READ
	SEC_GENERIC_WRITE
	SEC_GROUP_FROM_PARENT
	SEC_MASK_FLAGS
	SEC_MASK_GENERIC
	SEC_MASK_INVALID
	SEC_MASK_SPECIFIC
	SEC_MASK_STANDARD
	SEC_OWNER_FROM_PARENT
	SEC_PRIV_ADD_USERS
	SEC_PRIV_ADD_USERS_BIT
	SEC_PRIV_BACKUP
	SEC_PRIV_BACKUP_BIT
	SEC_PRIV_CHANGE_NOTIFY
	SEC_PRIV_CHANGE_NOTIFY_BIT
	SEC_PRIV_CREATE_GLOBAL
	SEC_PRIV_CREATE_GLOBAL_BIT
	SEC_PRIV_CREATE_PAGEFILE
	SEC_PRIV_CREATE_PAGEFILE_BIT
	SEC_PRIV_DEBUG
	SEC_PRIV_DEBUG_BIT
	SEC_PRIV_DISK_OPERATOR
	SEC_PRIV_DISK_OPERATOR_BIT
	SEC_PRIV_ENABLE_DELEGATION
	SEC_PRIV_ENABLE_DELEGATION_BIT
	SEC_PRIV_IMPERSONATE
	SEC_PRIV_IMPERSONATE_BIT
	SEC_PRIV_INCREASE_BASE_PRIORITY
	SEC_PRIV_INCREASE_BASE_PRIORITY_BIT
	SEC_PRIV_INCREASE_QUOTA
	SEC_PRIV_INCREASE_QUOTA_BIT
	SEC_PRIV_INVALID
	SEC_PRIV_LOAD_DRIVER
	SEC_PRIV_LOAD_DRIVER_BIT
	SEC_PRIV_MACHINE_ACCOUNT
	SEC_PRIV_MACHINE_ACCOUNT_BIT
	SEC_PRIV_MANAGE_VOLUME
	SEC_PRIV_MANAGE_VOLUME_BIT
	SEC_PRIV_PRINT_OPERATOR
	SEC_PRIV_PRINT_OPERATOR_BIT
	SEC_PRIV_PROFILE_SINGLE_PROCESS
	SEC_PRIV_PROFILE_SINGLE_PROCESS_BIT
	SEC_PRIV_REMOTE_SHUTDOWN
	SEC_PRIV_REMOTE_SHUTDOWN_BIT
	SEC_PRIV_RESTORE
	SEC_PRIV_RESTORE_BIT
	SEC_PRIV_SECURITY
	SEC_PRIV_SECURITY_BIT
	SEC_PRIV_SHUTDOWN
	SEC_PRIV_SHUTDOWN_BIT
	SEC_PRIV_SYSTEMTIME
	SEC_PRIV_SYSTEMTIME_BIT
	SEC_PRIV_SYSTEM_ENVIRONMENT
	SEC_PRIV_SYSTEM_ENVIRONMENT_BIT
	SEC_PRIV_SYSTEM_PROFILE
	SEC_PRIV_SYSTEM_PROFILE_BIT
	SEC_PRIV_TAKE_OWNERSHIP
	SEC_PRIV_TAKE_OWNERSHIP_BIT
	SEC_PRIV_UNDOCK
	SEC_PRIV_UNDOCK_BIT
	SEC_REG_CREATE_LINK
	SEC_REG_CREATE_SUBKEY
	SEC_REG_ENUM_SUBKEYS
	SEC_REG_NOTIFY
	SEC_REG_QUERY_VALUE
	SEC_REG_SET_VALUE
	SEC_RIGHTS_DIR_ALL
	SEC_RIGHTS_DIR_EXECUTE
	SEC_RIGHTS_DIR_READ
	SEC_RIGHTS_DIR_WRITE
	SEC_RIGHTS_FILE_ALL
	SEC_RIGHTS_FILE_EXECUTE
	SEC_RIGHTS_FILE_READ
	SEC_RIGHTS_FILE_WRITE
	SEC_RIGHTS_PRIV_BACKUP
	SEC_RIGHTS_PRIV_RESTORE
	SEC_SACL_AUTO_INHERIT
	SEC_STD_ALL
	SEC_STD_DELETE
	SEC_STD_READ_CONTROL
	SEC_STD_REQUIRED
	SEC_STD_SYNCHRONIZE
	SEC_STD_WRITE_DAC
	SEC_STD_WRITE_OWNER
	STANDARD_RIGHTS_ALL_ACCESS
	STANDARD_RIGHTS_EXECUTE_ACCESS
	STANDARD_RIGHTS_MODIFY_ACCESS
	STANDARD_RIGHTS_READ_ACCESS
	STANDARD_RIGHTS_REQUIRED_ACCESS
	STANDARD_RIGHTS_WRITE_ACCESS
);

our $VERSION = '0.01';

sub AUTOLOAD {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.

    my $constname;
    our $AUTOLOAD;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    croak "&Samba::Security::Descriptor::constant not defined" if $constname eq 'constant';
    my ($error, $val) = constant($constname);
    if ($error) { croak $error; }
    {
	no strict 'refs';
	# Fixed between 5.005_53 and 5.005_61
#XXX	if ($] >= 5.00561) {
#XXX	    *$AUTOLOAD = sub () { $val };
#XXX	}
#XXX	else {
	    *$AUTOLOAD = sub { $val };
#XXX	}
    }
    goto &$AUTOLOAD;
}

require XSLoader;
XSLoader::load('Samba::Security::Descriptor', $VERSION);

# Preloaded methods go here.

sub new
{
    my $class = shift;

    my $self = {};
    bless ($self, $class);

    $self->init();

    return $self;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

Samba::Security::Descriptor - Perl module to manage samba security descriptors

=head1 SYNOPSIS

  use Samba::Security::Descriptor;
  my $sd = new Samba::Security::Descriptor();

  $sd->from_sddl($sddl);
  $sd->as_sddl();

=head1 DESCRIPTION

  This module implements an object interface

=head2 EXPORT

None by default.

=head2 Exportable constants

  BUILTIN_RID_ACCOUNT_OPERATORS
  BUILTIN_RID_ADMINISTRATORS
  BUILTIN_RID_AUTH_ACCESS
  BUILTIN_RID_BACKUP_OPERATORS
  BUILTIN_RID_CERT_SERV_DCOM_ACCESS
  BUILTIN_RID_CRYPTO_OPERATORS
  BUILTIN_RID_DISTRIBUTED_COM_USERS
  BUILTIN_RID_EVENT_LOG_READERS
  BUILTIN_RID_GUESTS
  BUILTIN_RID_INCOMING_FOREST_TRUST
  BUILTIN_RID_NETWORK_CONF_OPERATORS
  BUILTIN_RID_PERFLOG_USERS
  BUILTIN_RID_PERFMON_USERS
  BUILTIN_RID_POWER_USERS
  BUILTIN_RID_PRE_2K_ACCESS
  BUILTIN_RID_PRINT_OPERATORS
  BUILTIN_RID_RAS_SERVERS
  BUILTIN_RID_REMOTE_DESKTOP_USERS
  BUILTIN_RID_REPLICATOR
  BUILTIN_RID_SERVER_OPERATORS
  BUILTIN_RID_TS_LICENSE_SERVERS
  BUILTIN_RID_USERS
  DOMAIN_RID_ADMINISTRATOR
  DOMAIN_RID_ADMINS
  DOMAIN_RID_CERT_ADMINS
  DOMAIN_RID_DCS
  DOMAIN_RID_DOMAIN_MEMBERS
  DOMAIN_RID_ENTERPRISE_ADMINS
  DOMAIN_RID_ENTERPRISE_READONLY_DCS
  DOMAIN_RID_GUEST
  DOMAIN_RID_GUESTS
  DOMAIN_RID_KRBTGT
  DOMAIN_RID_LOGON
  DOMAIN_RID_POLICY_ADMINS
  DOMAIN_RID_RAS_SERVERS
  DOMAIN_RID_READONLY_DCS
  DOMAIN_RID_RODC_ALLOW
  DOMAIN_RID_RODC_DENY
  DOMAIN_RID_SCHEMA_ADMINS
  DOMAIN_RID_USERS
  KERB_ENCTYPE_AES128_CTS_HMAC_SHA1_96
  KERB_ENCTYPE_AES256_CTS_HMAC_SHA1_96
  KERB_ENCTYPE_DES_CBC_CRC
  KERB_ENCTYPE_DES_CBC_MD5
  KERB_ENCTYPE_RC4_HMAC_MD5
  LSA_POLICY_MODE_ALL
  LSA_POLICY_MODE_ALL_NT4
  LSA_POLICY_MODE_BATCH
  LSA_POLICY_MODE_DENY_BATCH
  LSA_POLICY_MODE_DENY_INTERACTIVE
  LSA_POLICY_MODE_DENY_NETWORK
  LSA_POLICY_MODE_DENY_REMOTE_INTERACTIVE
  LSA_POLICY_MODE_DENY_SERVICE
  LSA_POLICY_MODE_INTERACTIVE
  LSA_POLICY_MODE_NETWORK
  LSA_POLICY_MODE_PROXY
  LSA_POLICY_MODE_REMOTE_INTERACTIVE
  LSA_POLICY_MODE_SERVICE
  NT4_ACL_REVISION
  SD_REVISION
  SECINFO_DACL
  SECINFO_GROUP
  SECINFO_LABEL
  SECINFO_OWNER
  SECINFO_PROTECTED_DACL
  SECINFO_PROTECTED_SACL
  SECINFO_SACL
  SECINFO_UNPROTECTED_DACL
  SECINFO_UNPROTECTED_SACL
  SECURITY_ACL_REVISION_ADS
  SECURITY_ACL_REVISION_NT4
  SECURITY_DESCRIPTOR_REVISION_1
  SEC_ACE_FLAG_CONTAINER_INHERIT
  SEC_ACE_FLAG_FAILED_ACCESS
  SEC_ACE_FLAG_INHERITED_ACE
  SEC_ACE_FLAG_INHERIT_ONLY
  SEC_ACE_FLAG_NO_PROPAGATE_INHERIT
  SEC_ACE_FLAG_OBJECT_INHERIT
  SEC_ACE_FLAG_SUCCESSFUL_ACCESS
  SEC_ACE_FLAG_VALID_INHERIT
  SEC_ACE_INHERITED_OBJECT_TYPE_PRESENT
  SEC_ACE_OBJECT_TYPE_PRESENT
  SEC_ACE_TYPE_ACCESS_ALLOWED
  SEC_ACE_TYPE_ACCESS_ALLOWED_OBJECT
  SEC_ACE_TYPE_ACCESS_DENIED
  SEC_ACE_TYPE_ACCESS_DENIED_OBJECT
  SEC_ACE_TYPE_ALLOWED_COMPOUND
  SEC_ACE_TYPE_SYSTEM_ALARM
  SEC_ACE_TYPE_SYSTEM_ALARM_OBJECT
  SEC_ACE_TYPE_SYSTEM_AUDIT
  SEC_ACE_TYPE_SYSTEM_AUDIT_OBJECT
  SEC_ADS_CONTROL_ACCESS
  SEC_ADS_CREATE_CHILD
  SEC_ADS_DELETE_CHILD
  SEC_ADS_DELETE_TREE
  SEC_ADS_GENERIC_ALL
  SEC_ADS_GENERIC_ALL_DS
  SEC_ADS_GENERIC_EXECUTE
  SEC_ADS_GENERIC_READ
  SEC_ADS_GENERIC_WRITE
  SEC_ADS_LIST
  SEC_ADS_LIST_OBJECT
  SEC_ADS_READ_PROP
  SEC_ADS_SELF_WRITE
  SEC_ADS_WRITE_PROP
  SEC_DACL_AUTO_INHERIT
  SEC_DEFAULT_DESCRIPTOR
  SEC_DESC_DACL_AUTO_INHERITED
  SEC_DESC_DACL_AUTO_INHERIT_REQ
  SEC_DESC_DACL_DEFAULTED
  SEC_DESC_DACL_PRESENT
  SEC_DESC_DACL_PROTECTED
  SEC_DESC_DACL_TRUSTED
  SEC_DESC_GROUP_DEFAULTED
  SEC_DESC_OWNER_DEFAULTED
  SEC_DESC_RM_CONTROL_VALID
  SEC_DESC_SACL_AUTO_INHERITED
  SEC_DESC_SACL_AUTO_INHERIT_REQ
  SEC_DESC_SACL_DEFAULTED
  SEC_DESC_SACL_PRESENT
  SEC_DESC_SACL_PROTECTED
  SEC_DESC_SELF_RELATIVE
  SEC_DESC_SERVER_SECURITY
  SEC_DIR_ADD_FILE
  SEC_DIR_ADD_SUBDIR
  SEC_DIR_DELETE_CHILD
  SEC_DIR_LIST
  SEC_DIR_READ_ATTRIBUTE
  SEC_DIR_READ_EA
  SEC_DIR_TRAVERSE
  SEC_DIR_WRITE_ATTRIBUTE
  SEC_DIR_WRITE_EA
  SEC_FILE_ALL
  SEC_FILE_APPEND_DATA
  SEC_FILE_EXECUTE
  SEC_FILE_READ_ATTRIBUTE
  SEC_FILE_READ_DATA
  SEC_FILE_READ_EA
  SEC_FILE_WRITE_ATTRIBUTE
  SEC_FILE_WRITE_DATA
  SEC_FILE_WRITE_EA
  SEC_FLAG_MAXIMUM_ALLOWED
  SEC_FLAG_SYSTEM_SECURITY
  SEC_GENERIC_ALL
  SEC_GENERIC_EXECUTE
  SEC_GENERIC_READ
  SEC_GENERIC_WRITE
  SEC_GROUP_FROM_PARENT
  SEC_MASK_FLAGS
  SEC_MASK_GENERIC
  SEC_MASK_INVALID
  SEC_MASK_SPECIFIC
  SEC_MASK_STANDARD
  SEC_OWNER_FROM_PARENT
  SEC_PRIV_ADD_USERS
  SEC_PRIV_ADD_USERS_BIT
  SEC_PRIV_BACKUP
  SEC_PRIV_BACKUP_BIT
  SEC_PRIV_CHANGE_NOTIFY
  SEC_PRIV_CHANGE_NOTIFY_BIT
  SEC_PRIV_CREATE_GLOBAL
  SEC_PRIV_CREATE_GLOBAL_BIT
  SEC_PRIV_CREATE_PAGEFILE
  SEC_PRIV_CREATE_PAGEFILE_BIT
  SEC_PRIV_DEBUG
  SEC_PRIV_DEBUG_BIT
  SEC_PRIV_DISK_OPERATOR
  SEC_PRIV_DISK_OPERATOR_BIT
  SEC_PRIV_ENABLE_DELEGATION
  SEC_PRIV_ENABLE_DELEGATION_BIT
  SEC_PRIV_IMPERSONATE
  SEC_PRIV_IMPERSONATE_BIT
  SEC_PRIV_INCREASE_BASE_PRIORITY
  SEC_PRIV_INCREASE_BASE_PRIORITY_BIT
  SEC_PRIV_INCREASE_QUOTA
  SEC_PRIV_INCREASE_QUOTA_BIT
  SEC_PRIV_INVALID
  SEC_PRIV_LOAD_DRIVER
  SEC_PRIV_LOAD_DRIVER_BIT
  SEC_PRIV_MACHINE_ACCOUNT
  SEC_PRIV_MACHINE_ACCOUNT_BIT
  SEC_PRIV_MANAGE_VOLUME
  SEC_PRIV_MANAGE_VOLUME_BIT
  SEC_PRIV_PRINT_OPERATOR
  SEC_PRIV_PRINT_OPERATOR_BIT
  SEC_PRIV_PROFILE_SINGLE_PROCESS
  SEC_PRIV_PROFILE_SINGLE_PROCESS_BIT
  SEC_PRIV_REMOTE_SHUTDOWN
  SEC_PRIV_REMOTE_SHUTDOWN_BIT
  SEC_PRIV_RESTORE
  SEC_PRIV_RESTORE_BIT
  SEC_PRIV_SECURITY
  SEC_PRIV_SECURITY_BIT
  SEC_PRIV_SHUTDOWN
  SEC_PRIV_SHUTDOWN_BIT
  SEC_PRIV_SYSTEMTIME
  SEC_PRIV_SYSTEMTIME_BIT
  SEC_PRIV_SYSTEM_ENVIRONMENT
  SEC_PRIV_SYSTEM_ENVIRONMENT_BIT
  SEC_PRIV_SYSTEM_PROFILE
  SEC_PRIV_SYSTEM_PROFILE_BIT
  SEC_PRIV_TAKE_OWNERSHIP
  SEC_PRIV_TAKE_OWNERSHIP_BIT
  SEC_PRIV_UNDOCK
  SEC_PRIV_UNDOCK_BIT
  SEC_REG_CREATE_LINK
  SEC_REG_CREATE_SUBKEY
  SEC_REG_ENUM_SUBKEYS
  SEC_REG_NOTIFY
  SEC_REG_QUERY_VALUE
  SEC_REG_SET_VALUE
  SEC_RIGHTS_DIR_ALL
  SEC_RIGHTS_DIR_EXECUTE
  SEC_RIGHTS_DIR_READ
  SEC_RIGHTS_DIR_WRITE
  SEC_RIGHTS_FILE_ALL
  SEC_RIGHTS_FILE_EXECUTE
  SEC_RIGHTS_FILE_READ
  SEC_RIGHTS_FILE_WRITE
  SEC_RIGHTS_PRIV_BACKUP
  SEC_RIGHTS_PRIV_RESTORE
  SEC_SACL_AUTO_INHERIT
  SEC_STD_ALL
  SEC_STD_DELETE
  SEC_STD_READ_CONTROL
  SEC_STD_REQUIRED
  SEC_STD_SYNCHRONIZE
  SEC_STD_WRITE_DAC
  SEC_STD_WRITE_OWNER
  STANDARD_RIGHTS_ALL_ACCESS
  STANDARD_RIGHTS_EXECUTE_ACCESS
  STANDARD_RIGHTS_MODIFY_ACCESS
  STANDARD_RIGHTS_READ_ACCESS
  STANDARD_RIGHTS_REQUIRED_ACCESS
  STANDARD_RIGHTS_WRITE_ACCESS

=head1 SEE ALSO

    Samba::Security::AccessControlEntry

=head1 AUTHOR

Samuel Cabrero, E<lt>scabrero@zentyal.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2013 by Zentyal S.L.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License, version 2, as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

=cut
